import { makeError, VElement, VTextNode } from '@textbus/core';
import { ref, h, getCurrentInstance, onMounted, onUpdated, onUnmounted } from 'vue';
import { DomAdapter } from '@textbus/platform-browser';

const adapterError = makeError('VueAdapter');
class VueAdapter extends DomAdapter {
    constructor(components, mount) {
        super({
            createCompositionNode: (compositionState, updateNativeCompositionNode) => {
                return new VElement('span', {
                    style: {
                        textDecoration: 'underline'
                    },
                    ref: updateNativeCompositionNode
                }, [
                    new VTextNode(compositionState.text)
                ]);
            },
            getParentNode(node) {
                return node.parentNode;
            },
            getChildNodes(parentElement) {
                return Array.from(parentElement.childNodes);
            },
            isNativeElementNode(node) {
                return node instanceof HTMLElement;
            },
            getChildByIndex(parentElement, index) {
                return parentElement.childNodes[index];
            },
            getAndUpdateSlotRootNativeElement(vElement, update) {
                const currentRef = vElement.attrs.get('ref');
                if (currentRef) {
                    vElement.attrs.set('ref', (v) => {
                        update(v);
                        if (typeof currentRef === 'function') {
                            currentRef(v);
                        }
                        else if (!currentRef.value) {
                            currentRef.value = v;
                        }
                    });
                }
                else {
                    vElement.attrs.set('ref', update);
                }
            },
            componentRender: (component) => {
                const comp = this.components[component.name] || this.components['*'];
                if (comp) {
                    let rootRef = this.componentRefs.get(component);
                    if (!rootRef) {
                        rootRef = ref();
                        this.componentRefs.set(component, rootRef);
                    }
                    return h(comp, {
                        component,
                        rootRef,
                        key: component.id
                    });
                }
                throw adapterError(`cannot found view component \`${component.name}\`!`);
            },
            vElementToViewElement(vNode, children) {
                const props = Object.assign({}, (Array.from(vNode.attrs).reduce((a, b) => {
                    a[b[0]] = b[1];
                    return a;
                }, {})));
                if (vNode.classes.size) {
                    props.class = Array.from(vNode.classes).join(' ');
                }
                if (vNode.styles) {
                    props.style = Array.from(vNode.styles).reduce((a, b) => {
                        a[b[0]] = b[1];
                        return a;
                    }, {});
                }
                return h(vNode.tagName, props, ...children);
            }
        }, mount);
        // private compositionRef = ref<HTMLElement>()
        this.componentRefs = new WeakMap();
        this.components = {};
        // watchEffect(() => {
        //   this.compositionNode = this.compositionRef.value || null
        // })
        Object.keys(components).forEach(key => {
            const vueComponent = components[key];
            const setup = vueComponent.setup;
            const self = this;
            vueComponent.setup = function (props) {
                const component = props.component;
                const vueInstance = getCurrentInstance();
                const sub = component.changeMarker.onChange.subscribe(() => {
                    if (component.changeMarker.dirty) {
                        vueInstance.proxy.$forceUpdate();
                    }
                });
                onMounted(() => {
                    if (props.rootRef.value) {
                        self.componentRootElementCaches.set(component, props.rootRef.value);
                    }
                    else {
                        self.componentRootElementCaches.remove(component);
                    }
                });
                onUpdated(() => {
                    var _a;
                    const context = self.componentRendingStack[self.componentRendingStack.length - 1];
                    if (context === component) {
                        self.componentRendingStack.pop();
                    }
                    component.changeMarker.rendered();
                    self.onViewUpdated.next();
                    if (!(((_a = self.componentRefs.get(component)) === null || _a === void 0 ? void 0 : _a.value) instanceof HTMLElement)) {
                        // eslint-disable-next-line max-len
                        throw adapterError(`Component \`${component.name}\` is not bound to rootRef, you must bind rootRef to the root element node of the component view.`);
                    }
                });
                onUnmounted(() => {
                    sub.unsubscribe();
                });
                const result = setup(props);
                if (typeof result === 'function') {
                    return function () {
                        component.__slots__.forEach(i => self.renderedSlotCache.delete(i));
                        component.__slots__.length = 0;
                        self.componentRendingStack.push(component);
                        return result();
                    };
                }
                return result;
            };
            if (vueComponent.render) {
                const oldRender = vueComponent.render;
                vueComponent.render = function (context, ...args) {
                    context.component.__slots__.length = 0;
                    self.componentRendingStack.push(context.component);
                    return oldRender.apply(this, [context, ...args]);
                };
            }
            this.components[key] = vueComponent;
        });
    }
    copy() {
        document.execCommand('copy');
    }
}

export { VueAdapter };
